import LiteEditor from 'lite-editor';

export default () => {
  const liteEditorLoadPromise = new Promise((resolve) => {
    const loader = new ACMS.SyncLoader();
    if (ACMS.Admin.liteEditorEmojiPicker) {
      resolve();
      return;
    }
    loader.next(`${ACMS.Config.jsRoot}dest/lite-editor-emoji-picker-plugin.js`).load(() => {
      resolve();
    });
  });

  const applyLiteEditorField = () => {
    const liteEditor = document.querySelectorAll(ACMS.Config.LiteEditorMark);
    [].forEach.call(liteEditor, (item) => {
      if ($(item).hasClass('editing')) {
        return;
      }
      $(item).addClass('editing');
      new LiteEditor(item, ACMS.Config.LiteEditorFieldConf); // eslint-disable-line no-new
    });
  };

  applyLiteEditorField();
  ACMS.addListener('acmsAddUnit', (event) => {
    applyLiteEditorField();
    if (ACMS.Config.LiteEditorFeature === true) {
      const item = event.obj.item;
      const selectOptions = [];
      const $select = $('.js-extendTagSelect select', item);
      const selectedOption = $select.val();
      const selectName = $select.attr('name');
      const extendValue = $('[name^="text_extend_tag"]', item).val();
      const sourceModeTags = /^(ul|ol|dl|pre|blockquote|none|markdown|wysiwyg|table)/;

      $('.js-extendTagSelect option', item).each(function () {
        const tag = $(this).val();
        const opt = {
          value: tag,
          label: $(this).text(),
          extendLabel: $(this).data('tag_extend')
        };
        if (tag === 'wysiwyg') {
          opt.onSelect = (self) => {
            self.data.mode = 'html';
            self.data.hideBtns = true;
            self.data.showSource = true;
            self.update();
            ACMS.Dispatch.emoditor(self._getElementByQuery('[data-selector="lite-editor-source"]'));
          };
        } else if (tag === 'markdown') {
          opt.onSelect = (self) => {
            const editor = $(self._getElementByQuery('[data-selector="lite-editor-source"]')).data('emoditor');
            if (editor) {
              self.data.value = editor.getData();
              editor.destroy();
              $(self._getElementByQuery('[data-selector="lite-editor-source"]')).data('emoditor', null);
            }
            self.data.mode = 'markdown';
            self.data.disableEditorMode = true;
            self.data.hideBtns = false;
            self.data.showSource = true;
            self.update();
            $(self._getElementByQuery('[data-selector="lite-editor-source"]')).show();
          };
        } else if (tag.match(sourceModeTags)) {
          opt.onSelect = (self) => {
            const editor = $(self._getElementByQuery('[data-selector="lite-editor-source"]')).data('emoditor');
            if (editor) {
              self.data.value = editor.getData();
              editor.destroy();
              $(self._getElementByQuery('[data-selector="lite-editor-source"]')).data('emoditor', null);
            }
            self.data.mode = 'html';
            self.data.disableEditorMode = true;
            self.data.hideBtns = false;
            self.data.showSource = true;
            self.update();
            $(self._getElementByQuery('[data-selector="lite-editor-source"]')).show();
          };
        } else {
          opt.onSelect = (self) => {
            const emoditor = $(self._getElementByQuery('[data-selector="lite-editor-source"]')).data('emoditor');
            if (emoditor) {
              self.data.value = emoditor.getData();
              emoditor.destroy();
              $(self._getElementByQuery('[data-selector="lite-editor-source"]')).data('emoditor', null);
            }
            self.data.showSource = false;
            self.data.mode = 'html';
            self.data.disableEditorMode = false;
            self.data.hideBtns = false;
            self.update();
            self._getElementByQuery('[data-selector="lite-editor"]').innerHTML = self.data.value;
            $('.entryFormLiteEditor', item).show();
          };
        }
        selectOptions.push(opt);
      });

      $('.entryFormColumnBody', item).not('editing').each(function () {
        const $textarea = $('.entryFormTextarea', this);
        if ($textarea.length === 0) {
          return;
        }
        $(this).addClass('editing');
        const $selector = $("[name^='text_tag']", this);
        const tag = $selector.val();
        let sourceFirst = false;
        $textarea.attr('rows', '1');
        ACMS.Dispatch.Edit._media(item);
        if (tag && tag.match(sourceModeTags)) {
          sourceFirst = true;
        }

        const isMobile = (navigator.userAgent.indexOf('iPhone') > 0 &&
          navigator.userAgent.indexOf('iPad') === -1) ||
          navigator.userAgent.indexOf('iPod') > 0 ||
          navigator.userAgent.indexOf('Android') > 0;

        const liteEditorAry = [];

        $textarea.each((i, textarea) => {
          const editor = new LiteEditor(textarea, $.extend({}, ACMS.Config.LiteEditorConf, {
            selectOptions,
            selectedOption,
            selectName,
            extendValue,
            sourceFirst,
            classNames: {
              LiteEditor: 'entryFormLiteEditor',
              LiteEditorSource: 'entryFormTextarea',
              LiteEditorBtnGroup: 'acms-admin-btn-group acms-admin-btn-group-inline',
              LiteEditorBtn: 'acms-admin-btn',
              LiteEditorBtnActive: 'acms-admin-btn acms-admin-btn-active',
              LiteEditorBtnClose: '',
              LiteEditorTooltipInput: 'acms-admin-form-width-full'
            },
            btnPosition: 'bottom',
            mode: selectedOption === 'markdown' ? 'markdown' : 'html',
          }));
          liteEditorAry.push(editor);

          if (ACMS.Config.LiteEditorUseEmojiPicker && ACMS.Config.dbCharset === 'utf8mb4') {
            if (!isMobile) {
              liteEditorLoadPromise.then(() => {
                editor.registerButton(new ACMS.Admin.liteEditorEmojiPicker({ // eslint-disable-line new-cap
                  label: ACMS.Config.LiteEditorEmojiPickerLabel
                }));
              });
            }
          }
          if (sourceFirst) {
            editor.deactivateEditorMode();
          }
          ACMS.Dispatch.Edit._media(item);
          $(item).data('lite-editor', editor);
          $('.entryFormColumnBody', item).focus();
        });
        $('.js-extendTagSelect', item).remove();
        $('.editTextInsert', item).remove();


        setTimeout(() => {
          $('.lite-editor-select', item).change(function () {
            liteEditorAry.forEach((editor) => {
              // 自身は除外
              if (editor._getElementByQuery('.lite-editor-select') === $(this).get(0)) {
                return;
              }
              editor.e = {
                target: {
                  value: $(this).val()
                }
              };
              editor.changeOption();
            });
          });
        }, 100);
      });
    }
  });
};
